@@ -36,13 +36,16 @@ struct _MagnifierPrivate {
 	GtkWidget *canvas;
 	GdkWindow *root;
 	GdkDrawable *cursor;
-        int cursor_default_size;
+        int cursor_default_size_x;
+        int cursor_default_size_y;
 	gboolean crosswire;
 	GdkBitmap *cursor_mask;
 	int cursor_x;
 	int cursor_y;
 	int cursor_hotspot_x;
 	int cursor_hotspot_y;
+        gboolean use_source_cursor;
+        GHashTable *cursorlist;
 };
 
 #ifdef __cplusplus
