@@ -90,6 +90,30 @@ magnifier_x_error_handler (Display	 *display,
 	return 0;
 }
 
+static void
+magnifier_warp_cursor_to_screen (Magnifier *magnifier)
+{
+	int x, y, unused_x, unused_y;
+	unsigned int mask;
+	Window root_return, child_return;
+
+	if (!XQueryPointer (GDK_DISPLAY_XDISPLAY (magnifier->source_display), 
+			    GDK_WINDOW_XWINDOW (magnifier->priv->root), 
+			    &root_return,
+			    &child_return,
+			    &x, &y,
+			    &unused_x, &unused_y,
+			    &mask))
+	{
+		XWarpPointer (GDK_DISPLAY_XDISPLAY (magnifier->source_display),
+			      None,
+			      GDK_WINDOW_XWINDOW (magnifier->priv->root),
+			      0, 0, 0, 0,
+			      x, y);
+		XSync (GDK_DISPLAY_XDISPLAY (magnifier->source_display), FALSE);
+	}
+}
+
 static void
 magnifier_zoom_regions_mark_dirty (Magnifier *magnifier, GNOME_Magnifier_RectBounds rect_bounds)
 {
@@ -224,6 +248,7 @@ magnifier_set_cursor_pixmap_by_name (Magnifier *magnifier, const gchar *cursor_n
 	magnifier_set_cursor_from_pixbuf (magnifier, pixbuf);
 	if (pixbuf) gdk_pixbuf_unref (pixbuf);
 }
+
 gboolean
 magnifier_cursor_notify (GIOChannel *source, GIOCondition condition, gpointer data)
 {
@@ -621,6 +646,7 @@ magnifier_transform_cursor (Magnifier *magnifier)
 		if (magnifier->cursor_size_x)
 		{
 			size_x = magnifier->cursor_size_x;
+			size_y = magnifier->cursor_size_y;
 		}
 		else
 		{
@@ -631,9 +657,9 @@ magnifier_transform_cursor (Magnifier *magnifier)
 		}
 		gdk_drawable_get_size (magnifier->priv->cursor, &width, &height);
 		if ((size_x == width) && (size_y == height) 
-		    && (magnifier->cursor_color == 0xFF000000))
+		    && (magnifier->cursor_color == 0xFF000000)) {
 			return; /* nothing changes */
-
+		}
 		cgc = gdk_gc_new (cursor_pixmap);
 		mgc = gdk_gc_new (mask_pixmap);
 		cursor_pixbuf = gdk_pixbuf_get_from_drawable (NULL, cursor_pixmap,
@@ -663,6 +689,7 @@ magnifier_transform_cursor (Magnifier *magnifier)
 			magnifier->priv->w->window,
 			size_x, size_y,
 			-1);
+		if (!magnifier->priv->cursor) g_warning ("magnifier cursur pixmap is non-null");
 		magnifier->priv->cursor_mask = gdk_pixmap_new (
 			magnifier->priv->w->window,
 			size_x, size_y,
@@ -843,6 +870,7 @@ magnifier_set_property (BonoboPropertyBag *bag,
 				gdk_display_get_screen (
 					magnifier->source_display,
 					magnifier->source_screen_num));
+		magnifier_warp_cursor_to_screen (magnifier);
 		magnifier_get_display_rect_bounds (magnifier, &magnifier->source_bounds, FALSE);
 		break;
 	case MAGNIFIER_TARGET_DISPLAY_PROP:
@@ -1360,6 +1388,7 @@ magnifier_get_cursor (Magnifier *magnifier)
 		{
 		    magnifier_set_cursor_from_pixbuf (magnifier, pixbuf);
 		    g_object_unref (pixbuf);
+		    magnifier_transform_cursor (magnifier);
 		}
 		g_free (default_cursor_filename);
 	    }
