@@ -155,7 +155,6 @@ static float cps_max           = 0;
 static float nrr_max           = 0;
 static float update_nrr_max    = 0;
 static gboolean	reset_timing   = FALSE;
-static GdkGC *default_gc       = NULL;
 static gboolean timing_test    = FALSE;
 
 static guint pending_idle_handler = 0;
@@ -223,7 +222,6 @@ _debug_announce_rect (char *msg, GdkRectangle rect)
 }
 
 #ifdef TEST_XTST_CURSOR
-/** util stuff **/
 static Cursor *
 _get_x_cursors (Display *display)
 {
@@ -580,6 +578,19 @@ _rectangle_clip_to_bounds (GdkRectangle area,
 	return area;
 }
 
+static GdkRectangle
+zoom_region_clip_to_source (ZoomRegion *zoom_region,
+			    GdkRectangle area)
+{
+    GNOME_Magnifier_RectBounds *source_rect_ptr;
+    if (zoom_region && zoom_region->priv && zoom_region->priv->parent)
+    {
+	source_rect_ptr = &((Magnifier *)zoom_region->priv->parent)->source_bounds;
+	return _rectangle_clip_to_bounds (area, source_rect_ptr);
+    }
+    return area;
+}
+
 static GdkRectangle
 zoom_region_clip_to_exposed_target (ZoomRegion *zoom_region,
 				    GdkRectangle area)
@@ -924,10 +935,10 @@ zoom_region_paint_cursor (ZoomRegion *zoom_region,
 		}
 		if (zoom_region->priv->w->window != NULL)
 		{
-			if (default_gc == NULL)
-				default_gc = gdk_gc_new(zoom_region->priv->w->window);
+			if (zoom_region->priv->default_gc == NULL) 
+				zoom_region->priv->default_gc = gdk_gc_new(zoom_region->priv->w->window);
 			gdk_draw_drawable (zoom_region->priv->cursor_backing_pixels,
-				     default_gc,
+				     zoom_region->priv->default_gc,
 				     zoom_region->priv->w->window,
 				     rect.x,
 				     rect.y,
@@ -1003,18 +1014,20 @@ zoom_region_paint_border (ZoomRegion *zoom_region,
 			  GdkRectangle *area)
 {
 	GdkColor color;
-	/*
-	if (zoom_region->border_size > 0)
+	if ((zoom_region->border_size > 0) && (zoom_region->priv->w->window))
+	{
 		if (!zoom_region->priv->border_gc)
 		{
 			zoom_region->priv->border_gc = gdk_gc_new (zoom_region->priv->w->window);
-			color.red = (zoom_region->border_color & 0xFF0000) >> 16;
-			color.green = (zoom_region->border_color & 0xFF00) >> 8;
-			color.blue = zoom_region->border_color & 0xFF;
-#ifdef DEBUG
+			color.red = (zoom_region->border_color & 0xFF0000) >> 8;
+			color.green = (zoom_region->border_color & 0xFF00);
+			color.blue = (zoom_region->border_color & 0xFF) << 8;
+#ifdef DEBUG_BORDER
 			fprintf (stderr, "border color triple RGB=%d|%d|%d",
 				 color.red, color.green, color.blue);
 #endif
+			gdk_colormap_alloc_color (gdk_drawable_get_colormap (zoom_region->priv->w->window), 
+						  &color, TRUE, TRUE);
 			gdk_gc_set_foreground (zoom_region->priv->border_gc, &color);
 		}
 		gdk_draw_rectangle (zoom_region->priv->w->window,
@@ -1024,20 +1037,20 @@ zoom_region_paint_border (ZoomRegion *zoom_region,
 				    area->y,
 				    area->width,
 				    area->height);
-	*/
+	}
 }
 
 static void
 zoom_region_paint_pixmap (ZoomRegion *zoom_region,
 			  GdkRectangle *area)
 {
-	if (default_gc == NULL)
-		default_gc = gdk_gc_new(zoom_region->priv->w->window);
+	if (zoom_region->priv->default_gc == NULL) 
+		zoom_region->priv->default_gc = gdk_gc_new(zoom_region->priv->w->window);
 
 	if (zoom_region->priv->pixmap)
 	{
 		gdk_draw_drawable (zoom_region->priv->w->window,
-				   default_gc,
+				   zoom_region->priv->default_gc,
 				   zoom_region->priv->pixmap,
 				   area->x + zoom_region->priv->exposed_bounds.x1,
 				   area->y + zoom_region->priv->exposed_bounds.y1,
@@ -1557,11 +1570,11 @@ zoom_region_get_source_subwindow (ZoomRegion *zoom_region,
 
 			zoom_region->priv->source_drawable = gdk_pixmap_new (NULL, width, height, 24);
 
-			if (default_gc == NULL)
-				default_gc = gdk_gc_new(zoom_region->priv->w->window);
+			if (zoom_region->priv->default_gc == NULL)
+				zoom_region->priv->default_gc = gdk_gc_new(zoom_region->priv->w->window);
 
 			gdk_draw_image (zoom_region->priv->source_drawable,
-					default_gc,
+					zoom_region->priv->default_gc,
 					test_image,
 					0, 0,
 					0, 0,
@@ -1632,7 +1645,9 @@ zoom_region_update (ZoomRegion *zoom_region,
 	GdkPixbuf *subimage;
 	GdkRectangle source_rect;
 	DEBUG_RECT ("unclipped update rect", update_rect);
-	source_rect = zoom_region_clip_to_exposed_target (zoom_region, update_rect);
+	source_rect = zoom_region_clip_to_source (zoom_region, update_rect);
+	DEBUG_RECT ("clipped to source", source_rect);
+	source_rect = zoom_region_clip_to_exposed_target (zoom_region, source_rect);
 	DEBUG_RECT ("update rect clipped to exposed target", source_rect); 
 	subimage = zoom_region_get_source_subwindow (zoom_region, source_rect);
 
@@ -1660,12 +1675,12 @@ zoom_region_update (ZoomRegion *zoom_region,
 
 		zoom_region_post_process_pixbuf (zoom_region, subimage,
 						 zoom_region->priv->scaled_pixbuf);
-		if (default_gc == NULL)
-			default_gc = gdk_gc_new(zoom_region->priv->w->window);
+		if (zoom_region->priv->default_gc == NULL)
+			zoom_region->priv->default_gc = gdk_gc_new(zoom_region->priv->w->window);
 
 #ifndef USE_GDK_PIXBUF_RENDER_TO_DRAWABLE 
 		gdk_draw_pixbuf (zoom_region->priv->pixmap,
-				 default_gc,
+				 zoom_region->priv->default_gc,
 				 zoom_region->priv->scaled_pixbuf,
 				 0,
 				 0,
@@ -1679,7 +1694,7 @@ zoom_region_update (ZoomRegion *zoom_region,
 #else
 		gdk_pixbuf_render_to_drawable (zoom_region->priv->scaled_pixbuf,
 					       zoom_region->priv->pixmap,
-					       default_gc,
+					       zoom_region->priv->default_gc,
 					       0,
 					       0,
 					       paint_rect.x + zoom_region->priv->exposed_bounds.x1,
@@ -1785,7 +1800,9 @@ zoom_region_process_updates (gpointer data)
 			zoom_region_update (zoom_region,
 					    * (GdkRectangle *) last->data);
 			g_list_free (last);
+#ifdef DEBUG
 			fputs (".\n", stderr); /* debug output, means we actually did something. */
+#endif
 		}
 		return TRUE;
 	}
@@ -2099,11 +2116,13 @@ zoom_region_set_viewport (ZoomRegion *zoom_region,
 			  const GNOME_Magnifier_RectBounds *viewport)
 {
 	zoom_region->viewport = *viewport;
+#define DEBUG
 #ifdef DEBUG
 	fprintf (stderr, "setting viewport %d,%d - %d,%d\n",
 		 (int) viewport->x1, (int) viewport->y1,
 		 (int) viewport->x2, (int) viewport->y2);
 #endif
+#undef DEBUG
 	zoom_region_recompute_exposed_viewport (zoom_region);
 	zoom_region_align (zoom_region);
 	if (!zoom_region->priv->w) {
@@ -2136,11 +2155,14 @@ zoom_region_get_property (BonoboPropertyBag *bag,
 	ZoomRegion *zoom_region = user_data;
 	
 	switch (arg_id) {
+	case ZOOM_REGION_MANAGED_PROP:
+		BONOBO_ARG_SET_BOOLEAN (arg, zoom_region->is_managed);
+		break;
 	case ZOOM_REGION_INVERT_PROP:
 		BONOBO_ARG_SET_BOOLEAN (arg, zoom_region->invert);
 		break;
 	case ZOOM_REGION_SMOOTHSCROLL_PROP:
-		BONOBO_ARG_SET_INT (arg, zoom_region->smooth_scroll_policy);
+		BONOBO_ARG_SET_SHORT (arg, zoom_region->smooth_scroll_policy);
 		break;
 	case ZOOM_REGION_TESTPATTERN_PROP:
 		BONOBO_ARG_SET_BOOLEAN (arg, zoom_region->priv->test);
@@ -2158,7 +2180,7 @@ zoom_region_get_property (BonoboPropertyBag *bag,
 		BONOBO_ARG_SET_FLOAT (arg, zoom_region->yscale);
 		break;
 	case ZOOM_REGION_BORDERSIZE_PROP:
-		BONOBO_ARG_SET_INT (arg, zoom_region->border_size);
+		BONOBO_ARG_SET_LONG (arg, zoom_region->border_size);
 		break;
 	case ZOOM_REGION_XALIGN_PROP:
 		/* TODO: enums here */
@@ -2204,17 +2226,16 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 	ZoomRegion *zoom_region = user_data;
 	GNOME_Magnifier_RectBounds bounds;
 
-	fprintf (stderr, "Setting zoom region property\n");
-	
 	switch (arg_id) {
+	case ZOOM_REGION_MANAGED_PROP:
+		zoom_region->is_managed = BONOBO_ARG_GET_BOOLEAN (arg);
+		break;
 	case ZOOM_REGION_INVERT_PROP:
 		zoom_region->invert = BONOBO_ARG_GET_BOOLEAN (arg);
 		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_SMOOTHSCROLL_PROP:
-		zoom_region->smooth_scroll_policy = BONOBO_ARG_GET_INT (arg);
-		fprintf (stderr, "Setting scroll smoothness property to %d\n",
-			zoom_region->smooth_scroll_policy);
+		zoom_region->smooth_scroll_policy = BONOBO_ARG_GET_SHORT (arg);
 		break;
 	case ZOOM_REGION_SMOOTHING_PROP:
 		zoom_region->smoothing = BONOBO_ARG_GET_STRING (arg);
@@ -2243,14 +2264,14 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_BORDERSIZE_PROP:
-		zoom_region->border_size = BONOBO_ARG_GET_INT (arg);
+		zoom_region->border_size = BONOBO_ARG_GET_LONG (arg);
+	    g_message ("border size set to %d", zoom_region->border_size);
 		zoom_region_recompute_exposed_viewport (zoom_region);
 		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_BORDERCOLOR_PROP:
 		zoom_region->border_color =
 			BONOBO_ARG_GET_LONG (arg);
-		fprintf (stderr, "border color = %x\n", (unsigned) zoom_region->border_color);
 		if (zoom_region->priv->border_gc)
 			g_object_unref (zoom_region->priv->border_gc);
 		zoom_region->priv->border_gc = NULL;
@@ -2258,8 +2279,6 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 		break;
 	case ZOOM_REGION_XALIGN_PROP:
 		zoom_region->x_align_policy = BONOBO_ARG_GET_INT (arg);
-		fprintf (stderr, "Setting x-alignment property to %d\n",
-			zoom_region->x_align_policy);
 		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_YALIGN_PROP:
@@ -2268,7 +2287,6 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_VIEWPORT_PROP:
-		fprintf (stderr, "setting viewport property...\n");
 		bounds = BONOBO_ARG_GET_GENERAL (arg,
 						 TC_GNOME_Magnifier_RectBounds,
 						 GNOME_Magnifier_RectBounds,
@@ -2278,26 +2296,16 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 	case ZOOM_REGION_TIMING_TEST_PROP:
 		zoom_region->timing_iterations = BONOBO_ARG_GET_INT (arg);
 		timing_test = TRUE;
-		fprintf (stderr, "Setting timing iterations property to %d\n", zoom_region->timing_iterations);
 		break;
 	case ZOOM_REGION_TIMING_OUTPUT_PROP:
 		zoom_region->timing_output = BONOBO_ARG_GET_BOOLEAN (arg);
-		if (zoom_region->timing_output)
-			fprintf (stderr, "Setting timing output property to true\n");
-		else
-			fprintf (stderr, "Setting timing output property to false\n");
 		break;
 	case ZOOM_REGION_TIMING_PAN_RATE_PROP:
 		zoom_region->timing_pan_rate = BONOBO_ARG_GET_INT (arg);
 		timing_test = TRUE;
-		fprintf (stderr, "Setting timing pan rate property to %d\n", zoom_region->timing_pan_rate);
 		break;
 	case ZOOM_REGION_EXIT_MAGNIFIER:
 		zoom_region->exit_magnifier = BONOBO_ARG_GET_BOOLEAN (arg);
-		if (zoom_region->exit_magnifier)
-			fprintf (stderr, "Setting exit magnifier property to true\n");
-		else
-			fprintf (stderr, "Setting exit magnifier property to false\n");
 		break;
 	default:
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
@@ -2472,7 +2480,7 @@ impl_zoom_region_set_mag_factor (PortableServer_Servant servant,
 			BONOBO_OBJREF (
 				(Magnifier *) zoom_region->priv->parent), ev);
 	any = Bonobo_PropertyBag_getValue (
-		properties, "source-display-size", ev);
+		properties, "source-display-bounds", ev);
 	if (!BONOBO_EX (ev))
 		zoom_region->priv->source_area =
 			*((GNOME_Magnifier_RectBounds *) any->_value);
@@ -2546,11 +2554,12 @@ impl_zoom_region_mark_dirty (PortableServer_Servant servant,
 {
 	ZoomRegion *zoom_region =
 		ZOOM_REGION (bonobo_object_from_servant (servant));
-	DEBUG_RECT ("mark dirty", zoom_region_source_rect_clipto_view_bounds (
+	DEBUG_RECT ("mark dirty", zoom_region_rect_from_bounds (
 			    zoom_region, roi_dirty) );
 	zoom_region_update_pointer (zoom_region, FALSE);
-	zoom_region_queue_update (zoom_region, zoom_region_clip_to_target
-				  (zoom_region, zoom_region_rect_from_bounds (zoom_region, roi_dirty)));
+	zoom_region_queue_update (zoom_region, 
+	  zoom_region_clip_to_target (zoom_region, 
+	      zoom_region_rect_from_bounds (zoom_region, roi_dirty)));
 }
 
 static GNOME_Magnifier_RectBounds
@@ -2651,13 +2660,13 @@ zoom_region_properties_init (ZoomRegion *zoom_region)
 				 Bonobo_PROPERTY_WRITEABLE);
 
 	bonobo_arg_release (def);
-	def = bonobo_arg_new (BONOBO_ARG_INT);
-	BONOBO_ARG_SET_INT (def, GNOME_Magnifier_ZoomRegion_SCROLL_FASTEST);
+	def = bonobo_arg_new (BONOBO_ARG_SHORT);
+	BONOBO_ARG_SET_SHORT (def, GNOME_Magnifier_ZoomRegion_SCROLL_FASTEST);
 	
 	bonobo_property_bag_add (zoom_region->properties,
 				 "smooth-scroll-policy",
 				 ZOOM_REGION_SMOOTHSCROLL_PROP,
-				 BONOBO_ARG_INT,
+				 BONOBO_ARG_SHORT,
 				 def,
 				 "scrolling policy, slower versus faster",
 				 Bonobo_PROPERTY_READABLE |
@@ -2739,13 +2748,13 @@ zoom_region_properties_init (ZoomRegion *zoom_region)
 				 Bonobo_PROPERTY_WRITEABLE);
 
 	bonobo_arg_release (def);
-	def = bonobo_arg_new (BONOBO_ARG_INT);
-	BONOBO_ARG_SET_INT (def, 0);
+	def = bonobo_arg_new (BONOBO_ARG_LONG);
+	BONOBO_ARG_SET_LONG (def, 0);
 	
 	bonobo_property_bag_add (zoom_region->properties,
 				 "border-size",
 				 ZOOM_REGION_BORDERSIZE_PROP,
-				 BONOBO_ARG_INT,
+				 BONOBO_ARG_LONG,
 				 def,
 				 "size of zoom-region borders, in pixels",
 				 Bonobo_PROPERTY_READABLE |
@@ -2862,6 +2871,7 @@ zoom_region_private_init (ZoomRegionPrivate *priv)
 	GdkRectangle rect = {0, 0, 0, 0};
 	priv->parent = NULL;
 	priv->w = NULL;
+	priv->default_gc = NULL;
 	priv->q = NULL;
 	priv->scaled_pixbuf = NULL;
 	priv->source_pixbuf_cache = NULL;
@@ -2903,11 +2913,6 @@ zoom_region_init (ZoomRegion *zoom_region)
 			    zoom_region_update_pointer_timeout,
 			    zoom_region,
 			    NULL);
-	/* to avoid reuse of an inappropriate default_gc, we NULL it when 
-	 * creating a new zoom region.  Better fix is in cvs HEAD,
-	 * but this one is smaller and more appropriate for 'stable' branch
-	 */
-	default_gc = NULL;
 }
 
 ZoomRegion *
